#ifndef __MULTIREFPTR_H__
#define __MULTIREFPTR_H__


// Written by Poul
// Modified by Ami

// Warning !!!!!!! Destructor is non-virtual !!!!!!!
// If you want to use pointer to derived class
// with additional data members of something as
// strange as this - please call us. Poul.


template<class DATA,class COUNT>
class BaseMultiRefPtr
{
protected:
	DATA* m_pData;
	COUNT* m_pRefCount;

protected:

	void Copy(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr);
	void Delete();
	
public:
	BaseMultiRefPtr(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr){Copy(SourceBaseMultiRefPtr);}
	BaseMultiRefPtr(const DATA* const pSourceData=NULL);
	~BaseMultiRefPtr(){Delete();}

	BaseMultiRefPtr<DATA,COUNT>& operator=(const BaseMultiRefPtr<DATA,COUNT> &SourceBaseMultiRefPtr);
	BaseMultiRefPtr<DATA,COUNT>& operator=(const DATA* const pSourceData);
	
	BOOL operator==(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr) const;
	BOOL operator!=(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr) const{return(!(*this==SourceBaseMultiRefPtr));}

	DATA& operator*(); 	
	const DATA& operator*() const;
	DATA* const operator->();
	const DATA* const operator->() const;

	DATA* const GetPtr(){return m_pData;}

	BOOL IsNull() const;
};


template<class DATA,class COUNT>
inline BaseMultiRefPtr<DATA,COUNT>::BaseMultiRefPtr(const DATA* const SourceDataPtr) :	m_pData(const_cast<DATA*>(SourceDataPtr)),m_pRefCount(NULL)
{
	if(m_pData!=NULL)
		if((m_pRefCount=new COUNT(1))==NULL)
			AfxThrowMemoryException();
}


template<class DATA,class COUNT>
inline BaseMultiRefPtr<DATA,COUNT>& BaseMultiRefPtr<DATA,COUNT>::operator=(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr)
{
	if(this!=&SourceBaseMultiRefPtr)
	{
		Delete();
		Copy(SourceBaseMultiRefPtr);
	}

	return(*this);
}


template<class DATA,class COUNT>
inline BaseMultiRefPtr<DATA,COUNT>& BaseMultiRefPtr<DATA,COUNT>::operator=(const DATA* const pSourceData) 
{
	if(m_pData!=pSourceData)
	{
		Delete();

		m_pData=const_cast<DATA*>(pSourceData);

		if(m_pData!=NULL)
			if((m_pRefCount=new COUNT(1))==NULL)
				AfxThrowMemoryException();
	}

#ifdef _DEBUG
	if(m_pData!=NULL)
		ASSERT(m_pRefCount!=NULL);
	else
		ASSERT(m_pRefCount==NULL);
#endif

	return(*this);
}


template <class DATA,class COUNT>
inline BOOL BaseMultiRefPtr<DATA,COUNT>::operator==(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr) const
{
#ifdef _DEBUG
	if(m_pData==SourceBaseMultiRefPtr.m_pData)
		ASSERT(m_pRefCount==SourceBaseMultiRefPtr.m_pRefCount);
#endif
	
	return(m_pData==SourceBaseMultiRefPtr.m_pData);
}


template<class DATA,class COUNT>
inline DATA& BaseMultiRefPtr<DATA,COUNT>::operator*()
{
	ASSERT(m_pData!=NULL);
	
	return(*m_pData);
}


template<class DATA,class COUNT>
inline const DATA& BaseMultiRefPtr<DATA,COUNT>::operator*() const
{
	ASSERT(m_pData!=NULL);
	
	return(*m_pData);
}


template<class DATA,class COUNT>
inline DATA* const BaseMultiRefPtr<DATA,COUNT>::operator->()
{
	ASSERT(m_pData!=NULL);
	
	return(m_pData);
}


template<class DATA,class COUNT>
inline const DATA* const BaseMultiRefPtr<DATA,COUNT>::operator->() const
{
	ASSERT(m_pData!=NULL);
	
	return(m_pData);
}


template<class DATA,class COUNT>
inline BOOL BaseMultiRefPtr<DATA,COUNT>::IsNull() const
{
#ifdef _DEBUG
	if(m_pData==NULL)
		ASSERT(m_pRefCount==NULL);
#endif

	return(m_pData==NULL);
}


template<class DATA,class COUNT>
inline void BaseMultiRefPtr<DATA,COUNT>::Copy(const BaseMultiRefPtr<DATA,COUNT>& SourceBaseMultiRefPtr)
{
	m_pData=SourceBaseMultiRefPtr.m_pData;
	m_pRefCount=SourceBaseMultiRefPtr.m_pRefCount;
	if(SourceBaseMultiRefPtr.m_pRefCount!=NULL)
	{
		ASSERT(m_pData!=NULL);
		
		++*m_pRefCount;
	}
#ifdef _DEBUG
	else
		ASSERT(SourceBaseMultiRefPtr.m_pData==NULL);
#endif
}


template<class DATA,class COUNT>
inline void BaseMultiRefPtr<DATA,COUNT>::Delete()
{
	if(m_pRefCount!=NULL)
	{
		ASSERT(m_pRefCount!=NULL);
		if(--*m_pRefCount==0)
		{
			delete m_pRefCount;
			delete m_pData;
		}

		m_pRefCount=NULL;
		m_pData=NULL;
	}
#ifdef _DEBUG
	else
		ASSERT(m_pData==NULL);
#endif
}

//*****************************************************************
template<class DATA,class COUNT>
inline void ThrowMemoryExceptionIfNull(const BaseMultiRefPtr<DATA,COUNT>& p)
{
	if (p.IsNull())
		AfxThrowMemoryException();
}

/////////////////////////////////////////////////////////////////////


template <class DATA>
class MultiRefPtr : public BaseMultiRefPtr<DATA,int>
{
public:
	MultiRefPtr(const MultiRefPtr<DATA>& SourceMultiRefPtr) : BaseMultiRefPtr<DATA,int>(SourceMultiRefPtr){}
	MultiRefPtr(const DATA* const pSourceData=NULL) : BaseMultiRefPtr<DATA,int>(pSourceData){}

	MultiRefPtr<DATA>& operator=(const MultiRefPtr<DATA> &SourceMultiRefPtr){return((MultiRefPtr<DATA>&)(BaseMultiRefPtr<DATA,int>::operator=(SourceMultiRefPtr)));}
	MultiRefPtr<DATA>& operator=(const DATA* const pSourceData){return((MultiRefPtr<DATA>&)(BaseMultiRefPtr<DATA,int>::operator=(pSourceData)));}

	BOOL operator==(const MultiRefPtr<DATA>& SourceMultiRefPtr) const{return(BaseMultiRefPtr<DATA,int>::operator==(SourceMultiRefPtr));}
	BOOL operator!=(const MultiRefPtr<DATA>& SourceMultiRefPtr) const{return(!(*this==SourceMultiRefPtr));}
};


#endif